@@ -3,7 +3,7 @@ __metaclass__ = type
 
 from units.compat import unittest
 from units.mock.loader import DictDataLoader
-from units.compat.mock import MagicMock
+from unittest.mock import MagicMock
 
 from ansible.template import Templar
 from ansible import errors
@@ -50,8 +50,8 @@ class TestConditional(unittest.TestCase):
 
     def test_undefined(self):
         when = [u"{{ some_undefined_thing }}"]
-        self.assertRaisesRegexp(errors.AnsibleError, "The conditional check '{{ some_undefined_thing }}' failed",
-                                self._eval_con, when, {})
+        self.assertRaisesRegex(errors.AnsibleError, "The conditional check '{{ some_undefined_thing }}' failed",
+                               self._eval_con, when, {})
 
     def test_defined(self):
         variables = {'some_defined_thing': True}
@@ -100,12 +100,12 @@ class TestConditional(unittest.TestCase):
 
         when = [u"some_dict.some_dict_key1 == hostvars['host3']"]
         # self._eval_con(when, variables)
-        self.assertRaisesRegexp(errors.AnsibleError,
-                                r"The conditional check 'some_dict.some_dict_key1 == hostvars\['host3'\]' failed",
-                                # "The conditional check 'some_dict.some_dict_key1 == hostvars['host3']' failed",
-                                # "The conditional check 'some_dict.some_dict_key1 == hostvars['host3']' failed.",
-                                self._eval_con,
-                                when, variables)
+        self.assertRaisesRegex(errors.AnsibleError,
+                               r"The conditional check 'some_dict.some_dict_key1 == hostvars\['host3'\]' failed",
+                               # "The conditional check 'some_dict.some_dict_key1 == hostvars['host3']' failed",
+                               # "The conditional check 'some_dict.some_dict_key1 == hostvars['host3']' failed.",
+                               self._eval_con,
+                               when, variables)
 
     def test_dict_undefined_values_bare(self):
         variables = {'dict_value': 1,
@@ -116,10 +116,10 @@ class TestConditional(unittest.TestCase):
 
         # raises an exception when a non-string conditional is passed to extract_defined_undefined()
         when = [u"some_defined_dict_with_undefined_values"]
-        self.assertRaisesRegexp(errors.AnsibleError,
-                                "The conditional check 'some_defined_dict_with_undefined_values' failed.",
-                                self._eval_con,
-                                when, variables)
+        self.assertRaisesRegex(errors.AnsibleError,
+                               "The conditional check 'some_defined_dict_with_undefined_values' failed.",
+                               self._eval_con,
+                               when, variables)
 
     def test_dict_undefined_values_is_defined(self):
         variables = {'dict_value': 1,
@@ -129,10 +129,10 @@ class TestConditional(unittest.TestCase):
                                                                  }}
 
         when = [u"some_defined_dict_with_undefined_values is defined"]
-        self.assertRaisesRegexp(errors.AnsibleError,
-                                "The conditional check 'some_defined_dict_with_undefined_values is defined' failed.",
-                                self._eval_con,
-                                when, variables)
+        self.assertRaisesRegex(errors.AnsibleError,
+                               "The conditional check 'some_defined_dict_with_undefined_values is defined' failed.",
+                               self._eval_con,
+                               when, variables)
 
     def test_is_defined(self):
         variables = {'some_defined_thing': True}
@@ -195,10 +195,10 @@ class TestConditional(unittest.TestCase):
                 u'hostvars["some_host"] is defined',
                 u"{{ compare_targets.triple }} is defined",
                 u"{{ compare_targets.quadruple }} is defined"]
-        self.assertRaisesRegexp(errors.AnsibleError,
-                                "The conditional check '{{ compare_targets.triple }} is defined' failed",
-                                self._eval_con,
-                                when, variables)
+        self.assertRaisesRegex(errors.AnsibleError,
+                               "The conditional check '{{ compare_targets.triple }} is defined' failed",
+                               self._eval_con,
+                               when, variables)
 
     def test_is_hostvars_host_is_defined(self):
         variables = {'hostvars': {'some_host': {}, }}
